\
\ Code for the DCC booster.
\
\ $ProjectVersion: 0.56 $
\

hex

\ Port mapping

0 pin-b thermal              \ H bridge thermal sensor (in)
1 pin-b presence             \ Presence detected (out)
2 pin-b inversion            \ Inversion active (out)
3 pin-b disconnection        \ Disconnection (out)
4 pin-b dcc                  \ DCC signal (in)
5 pin-b jumper               \ Inversion allowed jumper (in)
6 pin-b loco-detected        \ Engine detected on track (in)
7 pin-b short-detected       \ Short-circuit detected (in)
1 pin-c brake                \ H bridge BRAKE (out)
2 pin-c pwm                  \ H bridge PWM (out)

variable inversion-mode      \ Is inversion enabled?

\ Enable and disable output (act and signal)

: enable-output ( -- ) brake low pwm high disconnection low ;

: disable-output ( -- ) brake high pwm low disconnection high ;

\ Timer routines. At 10MhZ, with a prescaler of 2, each tick represent 0.8s.

: 50s ( -- )
    $c1 tmr0 !                        \ ($100-$c1)*0.8s = 50,4s
    t0if intcon bit-clr               \ Clear overflow bit
    begin t0if intcon bit-set? until  \ Wait for overflow bit to be set
;

variable timer-counter

: 300s ( -- )
    6 timer-counter v-for
      50s
    timer-counter v-next
;

\ Toggle inversion signal

: toggle-inversion ( -- ) inversion toggle ;

: handle-shortcut ( -- )
    \ Toggle inversion if inversion is allowed
    inversion-mode @ if toggle-inversion then
    \ Wait for 50s
    50s
    \ If shortcut is gone, exit
    short-detected high? /if exit then
    \ We are still in shortcut condition, disable output for 300s
    disable-output 300s enable-output
;

\ Maintain output disabled as long as thermal alert is on

: handle-thermal-alert ( -- )
    disable-output
    begin 300s thermal high? while repeat
    enable-output
;

\ If there is a thermal alert or a shortcut, handle them

: mainloop ( -- )
    begin
	thermal high? if handle-thermal-alert then
	short-detected high? if handle-shortcut then
    again ;

\ Initialization

: init ( -- )
    $00 portb !                              \ Clear portb output latches
    $f1 trisb !                              \ Port b 1, 2 and 3 as outputs
    jumper high? inversion-mode !            \ Inversion jumper
    $00 portc !                              \ Clear portc output latches
    $f9 trisc !                              \ Port c 1 and 2 as outputs
    $00 option_reg !                         \ CLKOUT, prescaler(tmr0)=2
    enable-output
;

\ Main program
main
: main init mainloop ;

\ Configuration word

fosc-hs set-fosc    \ High-speed oscillator
